@@ -5,7 +5,7 @@ import six
 
 from certbot.plugins import common
 
-REDIRECT_DIRECTIVES = ['return', 'rewrite']
+ADD_HEADER_DIRECTIVE = 'add_header'
 
 class Addr(common.Addr):
     r"""Represents an Nginx address, i.e. what comes after the 'listen'
@@ -29,6 +29,8 @@ class Addr(common.Addr):
     :param str port: port number or "\*" or ""
     :param bool ssl: Whether the directive includes 'ssl'
     :param bool default: Whether the directive includes 'default_server'
+    :param bool default: Whether this is an IPv6 address
+    :param bool ipv6only: Whether the directive includes 'ipv6only=on'
 
     """
     UNSPECIFIED_IPV4_ADDRESSES = ('', '*', '0.0.0.0')
@@ -82,12 +84,14 @@ class Addr(common.Addr):
                 port = tup[2]
 
         # The rest of the parts are options; we only care about ssl and default
-        while len(parts) > 0:
+        while parts:
             nextpart = parts.pop()
             if nextpart == 'ssl':
                 ssl = True
             elif nextpart == 'default_server':
                 default = True
+            elif nextpart == 'default':
+                default = True
             elif nextpart == "ipv6only=on":
                 ipv6only = True
 
@@ -116,7 +120,7 @@ class Addr(common.Addr):
     def __repr__(self):
         return "Addr(" + self.__str__() + ")"
 
-    def __hash__(self):
+    def __hash__(self):  # pylint: disable=useless-super-delegation
         # Python 3 requires explicit overridden for __hash__
         # See certbot-apache/certbot_apache/obj.py for more information
         return super(Addr, self).__hash__()
@@ -198,6 +202,14 @@ class VirtualHost(object):  # pylint: disable=too-few-public-methods
                      tuple(self.addrs), tuple(self.names),
                      self.ssl, self.enabled))
 
+    def has_header(self, header_name):
+        """Determine if this server block has a particular header set.
+        :param str header_name: The name of the header to check for, e.g.
+            'Strict-Transport-Security'
+        """
+        found = _find_directive(self.raw, ADD_HEADER_DIRECTIVE, header_name)
+        return found is not None
+
     def contains_list(self, test):
         """Determine if raw server block contains test list at top level
         """
@@ -212,15 +224,17 @@ class VirtualHost(object):  # pylint: disable=too-few-public-methods
         for a in self.addrs:
             if a.ipv6:
                 return True
+        return False
 
     def ipv4_enabled(self):
         """Return true if one or more of the listen directives in vhost are IPv4
         only"""
-        if self.addrs is None or len(self.addrs) == 0:
+        if not self.addrs:
             return True
         for a in self.addrs:
             if not a.ipv6:
                 return True
+        return False
 
     def display_repr(self):
         """Return a representation of VHost to be used in dialog"""
@@ -233,3 +247,19 @@ class VirtualHost(object):  # pylint: disable=too-few-public-methods
                 addrs=", ".join(str(addr) for addr in self.addrs),
                 names=", ".join(self.names),
                 https="Yes" if self.ssl else "No"))
+
+def _find_directive(directives, directive_name, match_content=None):
+    """Find a directive of type directive_name in directives. If match_content is given,
+       Searches for `match_content` in the directive arguments.
+    """
+    if not directives or isinstance(directives, six.string_types):
+        return None
+
+    # If match_content is None, just match on directive type. Otherwise, match on
+    # both directive type -and- the content!
+    if directives[0] == directive_name and \
+            (match_content is None or match_content in directives):
+        return directives
+
+    matches = (_find_directive(line, directive_name, match_content) for line in directives)
+    return next((m for m in matches if m is not None), None)
